#ifndef   __DATAVISA_H
#define   __DATAVISA_H

#if defined (Build_Library)
#define _CURVECLASS __declspec(dllexport)
#elif defined(Use_Library)
#define _CURVECLASS __declspec(dllimport)
#else
#define _CURVECLASS
#endif

//! gibt an nach welchem Kriterium die Kurve sortiert ist
typedef enum
{
  X_Sorted = 1, Y_Sorted, Z_Sorted, Incoming
} TOrder;

//! Fehlertype bei DatenBasis
typedef enum
{
  //! max. Anzahl speicherbarer Kurven ist erreicht
  MaxScanExceeded = 1,
  //! kein ausreich. Speicherplatz verfuegbar
  NoMemory
} TCFault;

// Benutzt in m_arscan.h 
typedef enum
{
  StandardFile = 1, ShiftedStandard, Tripel, Tupel, Bitmap, ShiftedBitmap,
		Tiff8, Tiff12, RAW8, RAW12, bkFile, dtnFile
} TSaveFormat;

// Benutzt in m_data.h/cpp 
typedef struct
{
  RECT  place;
  float zMin, zMax;
  char  zUnit[15];
  float ColorMap[256];
  int   nColorRange;
} TColorTable;

// Benutzt in m_xscan.h, m_data.h/cpp, m_arscan.cpp 
typedef struct
{
  float x, y;
  int   Ord;
} TKSystem[4];

//! Struktur eines Elements der Kurve
struct CPoint
{
  float Pt[ 3 ];
  BOOL Valid;
};

//! Klasse zum Verwalten einer Folge von Messwerten und zugehoeriger Parametern
class  _CURVECLASS TCurve
{
  public:

         //! Konstuktor Parameter gibt Anzahl der Messpunkte der Kurve an die
         //! 'vorlaeufig' geplant sind (kann spaeter noch geaendert werden)
         TCurve( int );
         ~TCurve( void );

         //! haengt einen Punkt(tripel) an Kurve an
         //! Rueckkehrcode false wenn fehler, wenn erfolgreich true
         BOOL PAdd( float &x, float &y, float &z );

         //! liest ein Wertetripel der Kurve aus
         //! Rueckkehrcode false wenn nicht moeglich sonst true
         BOOL PGet( float &x, float &y, float &z );

         //! wie PAdd (um schneller hintereinander Punkte anzuhaengen)
         //! Rueckkehrcode false bei Fehler, sonst true
         BOOL FastPAdd( float &x, float &y, float &z );

         //! wie PGet (fuer schnellen Zugriff)
         //! Rueckkehrcode false bei Fehler(Kurve hat keine Punkte,
         //!                     Speicherfehler,...)
         //! sonst true
         BOOL FastPGet( float &x, float &y, float &z );

         //! 'oeffnet' Kurve, macht Zugriffe auf Punkte erst moeglich
         //! Rueckkehrcode false bei Fehler sonst true
         BOOL FastOpen( void );

         //! 'schliesst' Kurve, kein Zugriff auf die Punkte moeglich
         //! Rueckkehrcode immer true
         BOOL FastClose( void );

         //! addiert val zum Y-wert des akt. Punktes der Kurve
         //! Rueckkehrcode false bei Fehler sonst true
         //! (wird nicht benutzt)
         BOOL ValueAdd( float &val );

         //! geht in der Kurve einen Punkt zurueck
         void BackStep( void );

         //! setzt Kurve auf Initialwerte zurueck
         //! Rueckkehrcode immer true
         BOOL New( void );

         //! markiert alle Punkte der Kurve als ungueltig, deren Y-wert<=(Level*100)
         //! Prozent von maximaler intensitaet der Kurve sind
         //! Rueckkehrcode false bei fehler sonst true
         //! (nur von GetPeakProperties benutzt)
         BOOL DeleteUnderGround( float Level );

         //! markiert alle Punkte der Kurve als ungueltig, laesst nur lokal.
         //! Max. uebrig (scheinbar nicht fertig impl.)
         //! Rueckkehrcode false bei Fehler sonst true
         //! (wird nicht benutzt)
         BOOL DeleteFlanks();

         //! legt alle Punkte(x,y,z) der Kurve in Datei mit angegeb. Namen
         //! (Parameter) ab
         //! Rueckkehrcode false bei fehler sonst true
         //! (wird nicht benutzt)
         BOOL Save( LPCSTR );

         //! speichert Kurve in ang. Datei(1.Parameter), 2. Param. gibt Art der
         //! Speicherung an(nur eine Option impl., diese speichert y-werte),
         //! 3. Param. nicht benutzt
         //! Rueckkehrcode false bei Fehler sonst true
         BOOL Save( LPCSTR, int, int );

         //! (nicht impl.)
         void SetLastPoint( void );

         //! setzt achse, nach welcher die Kurve geordnet ist
         //! (x,y,z oder messreihenf.)
         void SetOrder( TOrder order )
         { eOrder = order; };

         //! setzt internen zeiger auf den (pos). Wert der Kurve falls moeglich
         //! bei unzulaess. pos auf 1. bzw. letzten Wert
         //! Rueckkehrcode false wenn Kurve keine Punkte hat sonst true
         BOOL SetPP( int pos );

         //! liefert Id (Nr.) der Kurve als Rueckkehrcode
         //! (wird nicht benutzt)
         int GetId( void )
         { return Id; };

         //! liefert akt. Pos. des internen Zeiger der Kurve als Rueckkehrcode
         //! (wird nicht benutzt)
         int GetPP( void )
         { return ActualIdx; };

         //! liefert Anzahl der akt. im Speicher befindl. Kurven als Rueckkehrcode
         //! (wird nicht benutzt)
         int GetLastId( void )
         { return LastId; };

         //! liefert die Anzahl der Punkte der Kurve als Rueckkehrcode
         int GetPNumber( void )
         { return MaxPointIdx + 1; };

         //! berechnet eine durch den letzen Param. bestimmte Fkt
         //! (max. Intensitaet/Schwerpunkt)
         //! 1.Param(ausg.) x enthaelt nach aufruf je nach funktion berechneten Wert
         //! 2.Param(ausg.) y enthaelt nach aufruf je nach funktion berechneten Wert
         //! 3.Param(ausg.) z enthaelt nach aufruf je nach funktion berechneten Wert
         //! 4.Param(eing.) func ermoeglicht verschieden funktionen aufzurufen
         //!                (nur 2 implementiert)
         //!                  func=2  dann x,y,z tripel ermitteln, fuer das y (die
         //!                          Intensitaet) maximal ist
         //!                  func<>2 dann x,y,z Schwerpunkt der Kurve ermitteln
         //!                          (wird scheinbar nicht benutzt)
         //! Rueckkehrcode false wenn fehler sonst true
         BOOL GetGravityCenter( float&, float&, float&, int );

         //! liefert den in der Kurve vorhandenen dem Param 'search' naehsten X-wert
         //! (den indexwert) als Rueckkehrcode zurueck und den zugehoerigen Y-Wert
         //! sowie den abstand zwischen X-wert und search
         //! 1.param (eingabe) search in Kurve soll nach einem X-punkt gesucht
         //!                   werden, der zu search den geringsten abstand besitzt
         //! 2.Param (ausgabe) delta enthaelt nach aufruf den Abstand von search
         //!                   zum naehsten X-wert
         //! 3.Param (ausgabe) value enthaelt nach aufruf den zum naehsten X-wert
         //!                   gehoerigen Y-wert(Intensitaet)
         //! Rueckkehrcode ist der Index fuer den gilt X[Index] ist naehster X-wert
         //! zu search, wenn search im inneren des x-bereichs der Kurve ist sonst
         //! (wenn ausserhalb dieses bereiches oder bei fehler) den Wert 0
         //! voraussetzung ist, dass die X-werte der kurve von klein nach gross
         //! sortiert sind
         int GetValueByValue( float, float&, float& );

         //! nicht von diffr.gruppe auskommentiert
         //! GetReflection wird nirgends benutzt
         //! BOOL       GetReflection( int order, LPCPoint );

         //! liefert die Breite des 1.Bereiches, in dem alle y-werte des Tripels
         //! die Haelfte des Intensitaetbereichs(Y(max)-Y(min)) uebersteigen, sowie
         //! aus diesem Bereich x-wert,y-wert des Tripels fuer das Y-wert
         //! (Intensitaet) maximal ist
         //! 1. bereich bedeutet der am weitesten links liegende Bereich der obige
         //!     Eigenschaften erfuellt
         //! 1.Param(ausgabe) hw enthaelt nach aufruf breite des Bereichs
         //! 2.Param(ausgabe) pos enthaelt nach aufruf X-wert an der max. y-wert
         //!                      (intensitaet) in diesem Bereich
         //! 3.Param(ausgabe) intens enthaelt nach aufruf max.y-wert(intensitaet)
         //!                         in diesem Bereich
         //! Rueckkehrcode false wenn fehler sonst true
         //! anmerkung: dieser Bereich darf nicht bis ans ende der Kurve reichen
         //! d.h. kurve darf nicht monoton wachsend sein (sonst return false)
         //! (wird nicht benutzt)
         BOOL GetPeakProperties( float&, float&, float& );

         //! zuweisungsoperator fuer kurven, kopiert alle x,y,z,v werte sowie die
         //! max. und min. fuer x,y,z
         TCurve& operator=( const TCurve& );

         //! enthalten die min und max werte  der 3 achsen [0=x,1=y,2=z]
         //! bei kurve ohne punkte max[0..2]=-(min[0..2]=1000000.0)
         float Min[ 3 ], Max[ 3 ];

  private:

         //! handle auf speicherobjekt (x-werte)
         HGLOBAL hMemX;

         //! handle auf speicherobjekt (y-werte)
         HGLOBAL hMemY;

         //! handle auf speicherobjekt (z-werte)
         HGLOBAL hMemZ;

         //! handle auf speicherobjekt (valid)
         HGLOBAL hMemV;

         //! ordnung in der kurve vorliegt X_sorted/Y_sorted/Z_sorted oder Incoming
         TOrder eOrder;

         //! zeigt an ob kurve gerade im speicher fix. und somit zugriffe moeglich
         //! sind
         BOOL bStreamOpen;
         int Id;             // bezeichnet Kurve
         BOOL bEnd;           // Ende-Erkennung
         static int LastId;         // Anzahl der genutzten Kurven im Programm
         int MaxPointIdx;    // indiziert den letzten Eintrag
         int MaxMemIdx;      // indiziert den maximal moeglichen Eintrag
         int ActualIdx;      // indiziert den aktuellen Eintrag
         CPoint FAR * lpPX;  // Zeiger auf Datenfeld von X
};

typedef TCurve FAR * LPCurve;

//! klasse zum verwalten von mehreren Kurven (areascan)
class  _CURVECLASS TDataBase
{
  public:
         TDataBase( void );
         ~TDataBase( void );

         //! setzt internen Kurvenindex auf 0
         void Reset( void )
         { nActualIdx = 0; };

         //! bestimmt Anzahl der Kurven
         //! Rueckkehrcode = Anzahl der Kurven
         int GetCNumber( void )
         { return nMaxCurveIdx + 1; };

         //! liefert die Kurve als Rueckkehrcode deren nummer gleich dem
         //! uebergebenen Param. ist
         LPCurve GetCurve( int );

         //!? liefert als Rueckkehrcode die Scanreportcurve
         LPCurve GetScanReport( void )
         { return ScanReport; };

         //! liefert als Rueckkehrcode akt. Kurvenindex
         int GetCIdx( void )
         { return nActualIdx; };

         //! setzt Kurvenindex auf angegebenen wert(Parameter)
         void SetCIdx( int );

         //! fuegt eine Kurve der angegebenen Groesse zur Datenbasis hinzu
         //! Rueckkehrcode false wenn max. Kurvenanzahl erreicht sonst true
         BOOL AddCurve( int size );

         //! loescht alle Kurven der Datenbasis und setzt Indizes zurueck
         //! Rueckkehrcode immer true
         BOOL New( void );

         //! liefert als Rueckkehrcode die Anzahl maximal speicherbarer Kurven pro
         //! Datenbasis (default=999)
         int GetMaxScanNumber( void )
         {
                int cn = nMaxCurve - 1;
                return cn;
         };

         //! liefert als Rueckkehrcode den zuletzt aufgetretenen Fehler
         TCFault GetFault( void )
         { return eFault; };

         //! neu akk
         //! entfernt die letzte Kurve in der Datenbasis
         //! Rueckkehrcode FALSE wenn Datenbasis leer
         //! sonst TRUE
         BOOL DelCurve( void);
  private:

         //! Anzahl maximal speicherbarer Kurven (def.=1000)
         static const int nMaxCurve;

         //!  zuletzt aufgetretenen Fehler
         TCFault eFault;
         int nMaxCurveIdx;   // indiziert die letzte Kurve
         int nActualIdx;     // indiziert die aktuelle Kurve

         //! Liste von Kurven der Datenbasis
         LPCurve* CurveList;

         //! Kurve enthaelt Werte zu jeder Kurve der Datenbasis
         //! Omegawinkel,Thetawinkel(absoluten winkel) und je nach modus einen
         //! dritten wert (monitorintens., absorberpos.,integrale intens. ,...)
         LPCurve ScanReport;
};

typedef TDataBase FAR * LPDataBase;


typedef struct
{
	 double dThetaWindow, dMoveRelation, dOmegaMin, dOmegaMax;
   double dThetaMinFirst, dThetaMaxFirst, dThetaMinLast, dThetaMaxLast;
	 float fAngleRange, fAngleStep;
	 BOOL bPsd;
}
TDisplay;

typedef struct
{
    float omega,theta,intens;
}
TBMContens;

//! Bildschrim hat x und y Ausdehnung
typedef struct
{
    int x0, x1, dx, y0, y1, dy;
}
TScreen;

//! Koordinatensystem mit x,y,z
//! und jeweils einer Angabe 1 Pixel in x,y,z Richtung
//! entspricht *SF Koordinatensystemeinheit
//! und eingestellter Ausgabetyp
typedef struct
{
    double xMin, xMax, xSF;
    double yMin, yMax, ySF;
    double zMin, zMax, zSF;
    TScaleType xST, yST, zST;
}
TCoorSystem;

//! Aufzaezhlung Ausgabetypen/Darstellungsformen
typedef enum
{
    ReciprokeLatticeBitmap = 1 , MatrixBitmap = 2,
    Curve = 3 //! weil sowieso nicht implementiert, Number, BarGraph, Histogram
}
TOutputType;

//! Umrechnung Alt-/Neugrad...
#define DegToRad(val) ((val) * 0.01745329251994)
#define RadToDeg(val) ((val) / 0.01745329251994)

//! Umrechnung in Reziproke Einheiten
//! gendert, weil Umrechnung so korrekt und Darstellung nicht gespiegelt...
#define ArToRLx(_ome, _the) ((cos((_the)-(_ome))- cos(_ome)))
//!#define ArToRLx(_ome, _the) ((cos(_ome) - cos((_the) - (_ome))))
#define ArToRLy(_ome, _the) ((sin((_the)-(_ome))+ sin(_ome)))
//!#define ArToRLy(_ome, _the) ((sin(_ome) + sin((_the) - (_ome))))



//! Klasse um Bitmaps aus Kurvendaten zu errechnen und darzustellen
class TBitmapSource
{
    //! leider alles friend!, was in m_data.h
    friend class TPlotData;
    friend class TCurveShowParam;
    friend class TCurveFreeScal;
    friend class TCurveFreeScalColor;

public:

    //! Konstruktor
    TBitmapSource( TPlotData * );

    //! Destruktor
    virtual ~TBitmapSource();

    //! wird aufgerufen, wenn ein Bitmap nachgeladen werden soll (m_arscan.cpp)
    BOOL ProcessBitmapFile( HFILE );

    //! generiert den Header fuer ein Bitmap, setzt Koordinatensystem und Screen
    void FormatDBaseToBitmapSource( void );

    //! nicht implementiert
    void UpdateBitmapSource( void );

    //! False - keine Graustufendarstellung, True - Graustufendarstellung in Bitmaps
    //! wird im ini-File mit "swPalette" gesetzt
    BOOL bGrayScalePalette;

	 //! Zeiger auf "Daten"feld
	 //! hier wird zu jeder [x,y]-Pixelkoordinate der zugehoerigen
	 //! Theta-/OmegaWert gespeichert (nur RL-Bitmap)
	 TBMContens** daten;

	 //! TRUE - "Daten"feld existiert, FALSE - "Daten"feld existiert nicht
	 //! (bei RL-Bitmap wird zu jedem Bildpunkt Theta/Omega gespeichert)
	 BOOL dateniscreated;

protected:

	 void New( void );

	 //! laedt Bitmap Ressource - benoetigt?
	 void LoadBitmap( HDC );

	 //! berechnet Bytes pro Zeile*Hoehe des Bitmaps - Gesamtgroesse im Speicher
	 DWORD GetImageSize( void );

	 //! ruft bei RL_Bitmap DrawMeasurementArea()
	 //! berechnet Position der Anordnung der Farblegende
	 //! setzt Min,MaxIntensitaeten
	 //! ruft DrawColorTable()
	 void DrawBitmapFrame( HDC );

	 //! setzt Screen-Variablen, entsprechend der Bildschirmgroesse
	 //! setzt Units, Orientation, Scaling
	 void SetScreen( HDC, TScreen& );

	 //! berechnet entsprechende Intensitten
	 //! und gibt die Legende zu den Intensitten aus
	 void DrawColorTable( HDC, TColorTable& );

	 //! falls RL-Bitmapdarstellung, berechnet und zeichnet
	 //! das untersuchte Messgebiet im den reziproken Raum
	 void DrawMeasurementArea( HDC );

	 //! fuellt den Header eines Bitmaps mit Farbwerten der Palette
	 void FillBMInfoFromPalette( LOGPALETTE* );

	 //! Bitblock-Transfer von hMemDC (dem gegebenen Quell-DC)
	 //! zu diesem HDC (dem Ziel-DC) - entspricht der Darstellung eines Bitmaps
	 void DrawBitmap( HDC, int, int );

	 BITMAPFILEHEADER BMPFileHdr;

	 //! Zeiger auf ColorTable-Objekt
	 TColorTable ColorTable;

	 //! Zeiger auf zugehoeriges TPlotData-Objekt
	 TPlotData *DataWnd;

	 //! Zeiger auf zugehoerigen TCurveFreeScalColor-Dialog
	 TCurveFreeScalColor *UserFarb;

private:

	 //! function creates a device-dependent bitmap (DDB) from a device-independent
	 //! bitmap (DIB) and, optionally, sets the bitmap bits
	 HBITMAP CreateGDIObject( HDC );

	 //! Zwischenablageoperation, wenn Typ CF_DIB
	 HGLOBAL RenderDIB( void );

	 //! ruft CreatePaletteFromDIB(), ansonsten CreateDefaultPalette()
	 LOGPALETTE * GetPalette( void );

	 //! generiert Palette von DIB (Device independend Bitmap)
	 LOGPALETTE * CreatePaletteFromDIB( void );

	 //! generiert neue Palette (Graustufen oder Bunt)
	 LOGPALETTE * CreateDefaultPalette( void );

	 //! gibt Darstellungtyp zurueck
	 //! wird nicht benutzt
	 // TOutputType GetType( void );

	 //! um Intensitaetsabstufungen im User Falle zu generieren
	 //! ansonsten werden Defaultwerte gesetzt
	 //! fuellt ColorTable.ColorMap[cnt] mit Werten
	 void ColorDef(void);

	 //! setzt Variablenfelder farbe[] und ber[]
	 //! so dass ein linearer Verlauf abhaengig von Min-,MaxIntensitaet
	 void BereichDef(void);

	 //! gibt zu uebergebener Intensitaet entsprechende Intensitaetsfarbe zurueck
	 int GetColor( float );

	 //! generiere MatrixBitmap
	 void GenerateAngleSpaceBitmap(void);

	 //! genriere RL-Bitmap
	 void GenerateRLBitmap( void );

	 //! eigenes Koordinatensystem fuer Bitmapberechnung
	 TCoorSystem* BMCoorSys;

	 //! wichtig fuer Bitmap-Generierung
	 HGLOBAL hDIBInfo;

	 HGLOBAL hDIBData;

	 HGLOBAL hMemPal;

	 HPALETTE hPalette, hOldPalette;

	 HBITMAP hBitmap;

	 //! nicht mehr benoetigt, da GenerateAngleSpaceBitmap() geaendert
	 //! int nEnlargedLines;

	 //! int nAddedPoints;

	 //! Feld mit Intensitaetsabstufungen im Falle von "User Skalierung"
	 float ber[8];

	 //! Feld mit Farbabstufungen im Falle von "User Skalierung"
	 int farbe[8];

	 //! Speicherung der x-Bildpunkte, y-Bildpunkte
	 //! zwecks Vergleich, wenn neues "Daten"feld angelegt werden muss
	 int dxold,dyold;

	 //! Speicherung des alten Filenamens
	 //! zwecks Vergleich, wenn neues "Daten"feld angelegt werden muss
	 char Fileold[MaxString];
};




// Stellt die Funktionalitaet der Darstellung von Kurven und der Bewertung
// dieser Kurven bereit
class TPlotData : public TMDIWindow
{
	 friend class TCurveShowParam;
	 friend class TCurveFreeScal;
	 friend class TBitmapSource;
	 friend class TCurveFreeScalColor;

public:

	 //! Konstruktor
	 TPlotData( void );

	 //! Destruktor
	 virtual ~TPlotData( void );

	 // void Size( HWND, WPARAM, LPARAM );

	 //! fllt die Struktur lpPoints mit den Werten fr eine Kurve
	 //! um damit spter dann die Kurve darzustellen
	 void FormatCurveToPLine( TCurve *, LPPOINT, int & );

	 //! Paint wird von EvPaint aufgerufen und automatisch von Windows angefordert,
	 //! um den Inhalt des Fensters neu anzuzeigen
	 void Paint( HDC, PAINTSTRUCT * );

	 //! behandelt Kommando: linke Maustaste wurde gedrueckt
	 virtual void lButtonDown( WPARAM, LPARAM );

	 //! wenn linke Maustaste gedrueckt gehalten und dabei bewegt wird,
	 //! wird der Abstand zur Pos. des ersten Mausklicks berechnet
	 virtual void MouseMove( WPARAM, LPARAM );

	 //! gibt Handle zum zeichnen im Kindfenster zurck + linke Maustaste
	 //! als nicht gedrueckt kennzeichnen
	 virtual void lButtonUp( WPARAM, LPARAM );

	 //! wenn Darstellungstyp ein Bitmap, dann bereite Zwischenablage vor
	 void RenderFormat( WPARAM );

	 //! raeumt Zwischenablage auf, wenn Besitzer der Zwischenablage tot
	 void RenderAllFormats( WPARAM );

	 virtual BOOL New( void );

	 //! sichert Daten in der SecondCurve
	 //! m_main.cpp im Falle von cm_MoveScanReady
	 void PickUpData( LPARAM );

	 //! sichert Koordinatensystem in AltCoorsys
	 //! m_main.cpp im Falle von cm_MoveScanReady
	 void SetKSProperties( TKSProperties* );

	 //! fixiert die aktuelle Hauptkurve -> setzt Pointer von SecondCurve auf
	 //! MainCurve
	 void FreezeCurve( void );

	 //! setzt SecondCurve auf Initialwerte zurueck (loeschen aller Metripel)
	 //! zeichnet Scan-Fenster neu
	 void KillSecondCurve( void );

    //! speichert fixierte Kurve (SecondCurve) unter auszuwaehlendem Dateinamen
    //! mit Extension *.bk
    void SaveSecondCurve( void );

    //! setzt Variable bEnablePaint
    void EnablePaint( void )
    {
        bEnablePaint = TRUE;
    };

    //! setzt Variable bEnablePaint
    void DisablePaint( void )
    {
        bEnablePaint = FALSE;
    };

	 //! Abzissen/Scanachsebeschriftung
	 char abzisse[15];

protected:

    //! setzt den Filetyp, nach dem Laden von alten Daten
    void SetFileType(int);

    //! Funktion, die die bentigte Nachkommastellengenauigkeit
    //! berechnet und die ntigen Nachkommastellen zurckliefert
    int GetNachkommaStelle(TCoorSystem* pCoorSys,int x,int y);

    //! zeichnet Koord.System
    void DrawCoordinateSystem( HDC );

    //! laedt die Cursor-Ressource in den Speicher und vergibt ein Handle
    HCURSOR GetCursor( void )
    {
        return LoadCursor( NULL, IDC_CROSS );
    };

    //! Kopieren mit Hilfe der Zwischeablage - Typen: CF_DIB, CF_PALETTE
    void DoCopy( void );

    //! fuelle TKSystem mit Standardwerten
    virtual BOOL SetMeasurementArea( TKSystem & );

    //! setze neue Koordinatensystembereiche
    virtual void SetRanges( void );

    //! macht nix...
    virtual double GetThetaOffset( int )
    {
        return 0.0;
    };

    HFONT hFont;

    //! Zeiger auf zugehoeries Bitmap
    TBitmapSource* BSource;

    TCurve* MainCurve;

    TCurve* SecondCurve;

    //! gibt es ein 2. Koordinatensystem
    BOOL bSecondaryCoor;

    //! XY-Skalierung an/aus
    BOOL bXY_Scaling;

    //! gibt Darstellungstyp an
    TOutputType eOutputType;

    //! MinIntensitaet, MaxIntensitaet
    float LowerBound, UpperBound;

    //! erstes und alternatives Koordinatensystem
    TCoorSystem CoorSys, AltCoorSys;

    TScreen Screen;

    //! existiert Bitmap?
    BOOL bBitmapIsCreated;

    //! beiinhaltet den geladenen FileTyp
    int eLoadFormat;

	  //! FIX Fehler 39
	 TDisplay VisualDB;

private:

	 //! Darstellung zulassen oder nicht
    BOOL bEnablePaint;

    //! Mousebutton unten/oben
    BOOL bButtonDown;

    //! Koordinaten wohin mit der Maus geklickt wurde
    float xo, yo;

    //! beinhaltet Handle auf ein DC (WinAPI)
    HDC DragDC;
};


//! generiert Modaldialog "Einstellungen zur Darstellung"
class TCurveShowParam : public TModalDlg
{
public:
    TCurveShowParam(TMDIWindow *);
protected:

    //! Funktionen die den Dialog steuern...
    void Dlg_OnCommand( HWND, int, HWND, UINT );
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );
    BOOL CanClose( void );
    BOOL LeaveDialog( void );
    void Dlg_OnHScrollBar(HWND ,HWND ,UINT ,int );

private:

    //! den Dialog kann man uebr den Button "User Skalierung" aufrufen
    TModalDlg *FreeScaldlg;

    //! Groesse der y-Achse ueber Scrollbalken beeinflussbar
    int scrolldypos;

    //! Groesse der x-Achse ueber Scrollbalken beeinflussbar
    int scrolldxpos;

    //! fast alle Plotdaten werden gebraucht
    TPlotData *PlotWindow;

    //! eigenes Koordinatensystem/Alternatives
    TCoorSystem *CoorSys,*AltCoorSys;

    //! nicht mehr benoetigt
    // TOutputType bOutputToShow;

    //! einige Handles behalten...
    HWND hWndOutputT, hWndScaleT,hWndScrolldy,hWndScrolldx;
};

#endif
//DATAVISA_H
// __LastLine__