
#include "mc_812.h"
#include "cmd.h"
#include "../msim.h"
#include "../msimutil.h"

#include <iostream.h>
#include <iomanip.h>
#include <string.h>
#include <stdio.h>     // sprintf
#include <stdlib.h>    // rand, abs
#include <limits.h>
#include <dos.h>       // FP_SEG, FP_OFF
#include <math.h>      // ceil
#include <limits.h>

#define ETX  0x03
#define RTN  0x0d
#define CR   0x0d
#define LF   0x0a

#define endl "\n";

#define LOG(cmds) \
  if(m_log_stream && m_log_level>0) { (*m_log_stream) << cmds; }

#define COMMAND(cmds) \
  LOG("<cmd c='C812ISA'>" << cmds << "</cmd>" << endl);

#define LOGERROR_(cmds) \
  if(m_Controller->m_log_stream) { \
    (*m_Controller->m_log_stream) << \
      "<error c='C812ISA'>" << cmds << "</error>" << endl; \
  }


// defined in parser file (parser.y)
extern "C" void parse_command_string( char* buffer, MC_812* mc, int mode );


MC_812*  theMC_812;


MC_812::MC_812( )
{
  m_base_addr = 0xD000;
  m_ETX       = ETX;
  m_in_state  = false;
  m_in_char   = 0;
  m_in[ 0 ]   = RTN;
  m_in_pos    = 0;
  m_out[ 0 ]  = m_ETX;
  m_out[ 1 ]  = 0;
  m_out_pos   = 0;
  m_state     = 0xFFU;

  m_mode = dec_mode;    // DEFAULT?
  m_echo = false;

  m_motor[ 0 ] = new Motor( this );
  m_motor[ 1 ] = new Motor( this );
  m_motor[ 2 ] = new Motor( this );
  m_motor[ 3 ] = new Motor( this );

  SetDataAvailable( false );
  SetBusy( false );

  m_starttime = timeGetTime();

  m_log_stream = 0;
  m_log_level = 1;

  m_controller_busy_probability = 0;
}

MC_812::~MC_812( )
{
  delete m_motor[ 0 ];
  delete m_motor[ 1 ];
  delete m_motor[ 2 ];
  delete m_motor[ 3 ];
}

//inline
void MC_812::UpdateStatus( )
{
  updateSimulationStatusWindow( );
}

void MC_812::Error( int command, int axis, long parameter )
{
  LOG("<error c='C812ISA'>invalid command " << command << ", "
           << axis << ", " << parameter << "</error>");
}

void MC_812::AB( int axis )
{
  COMMAND( "abort " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> AB( );
  }
  else
  {
    m_motor[ axis-1 ] -> AB( );
  }
}

void MC_812::CP( int axis )
{
  COMMAND( "calc. point " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> CP( );
  }
  else
  {
    m_motor[ axis-1 ] -> CP( );
  }
}

void MC_812::FE( int axis, long direction )
{
  COMMAND( "find edge " << axis << ", " << direction );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> FE( direction );
  }
  else
  {
    m_motor[ axis-1 ] -> FE( direction );
  }
}

void MC_812::GH( int axis )
{
  COMMAND( "go home " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> GH( );
  }
  else
  {
    m_motor[ axis-1 ] -> GH( );
  }
}

void MC_812::GO( int axis )
{
  COMMAND( "motor go " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> GO( );
  }
  else
  {
    m_motor[ axis-1 ] -> GO( );
  }
}

void MC_812::MA( int axis, long position )
{
  COMMAND( "move absolute " << axis << ", " << position );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> MA( position );
  }
  else
  {
    m_motor[ axis-1 ] -> MA( position );
  }
}

void MC_812::MF( int axis )
{
  COMMAND( "motor off " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> MF( );
  }
  else
  {
    m_motor[ axis-1 ] -> MF( );
  }
}

void MC_812::MI( long point )
{
  COMMAND( "move to point, inc. " << point );
}

void MC_812::MN( int axis )
{
  COMMAND( "motor on " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> MN( );
  }
  else
  {
    m_motor[ axis-1 ] -> MN( );
  }
}

void MC_812::MP( long point )
{
  COMMAND( "move to point n " << point );
}

void MC_812::MR( int axis, long distance )
{
  COMMAND( "move relative " << axis << ", " << distance );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> MR( distance );
  }
  else
  {
    m_motor[ axis-1 ] -> MR( distance );
  }
}

void MC_812::ST( int axis )
{
  COMMAND( "stop motor " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> ST( );
  }
  else
  {
    m_motor[ axis-1 ] -> ST( );
  }
}





void MC_812::BR( long rate )
{
  COMMAND( "baud rate " << rate );
}

void MC_812::CF( long channel )
{
  COMMAND( "channel off " << channel );
}

void MC_812::CI( long channel )
{
  COMMAND( "channel input " << channel );
}

void MC_812::CN( long channel )
{
  COMMAND( "channel on " << channel );
}

void MC_812::CT( long channel )
{
  COMMAND( "channel out " << channel );
}

void MC_812::DB( int axis, long width )
{
  COMMAND( "dead band " << axis << ", " << width );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> DB( width );
  }
  else
  {
    m_motor[ axis-1 ] -> DB( width );
  }
}

void MC_812::DH( int axis )
{
  COMMAND( "define home " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> DH( );
  }
  else
  {
    m_motor[ axis-1 ] -> DH( );
  }
}

void MC_812::EN( )
{
  COMMAND( "echo on" );

  m_echo = true;
}

void MC_812::EF( )
{
  COMMAND( "echo off" );

  m_echo = false;
}

void MC_812::IA( long address )
{
  COMMAND( "ieee-488 address " << address );
}

void MC_812::LS( int axis, long limit )
{
  COMMAND( "limit set " << axis << ", " << limit );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> LS( limit );
  }
  else
  {
    m_motor[ axis-1 ] -> LS( limit );
  }
}

void MC_812::RT( )
{
  COMMAND( "reset system " );
}

void MC_812::SA( int axis, long acceleration )
{
  COMMAND( "set acceleration " << axis << ", " << acceleration );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> SA( acceleration );
  }
  else
  {
    m_motor[ axis-1 ] -> SA( acceleration );
  }
}

void MC_812::SD( int axis, long deceleration )
{
  COMMAND( "set deceleration " << axis << ", " << deceleration );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> SD( deceleration );
  }
  else
  {
    m_motor[ axis-1 ] -> SD( deceleration );
  }
}

void MC_812::SF( int axis )
{
  COMMAND( "slave off " << axis );

  // no forward, maintained by controller
}


void MC_812::SN( int axis, long master )
{
  COMMAND( "slave on " << axis << ", " << master );

  // no forward, maintained by controller
}


void MC_812::SG( int axis, long gain )
{
  COMMAND( "set gain " << axis << ", " << gain );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> SG( gain );
  }
  else
  {
    m_motor[ axis-1 ] -> SG( gain );
  }
}

void MC_812::SE( int axis, long dynamic_gain )
{
  COMMAND( "set dynamic gain " << axis << ", " << dynamic_gain );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> SE( dynamic_gain );
  }
  else
  {
    m_motor[ axis-1 ] -> SE( dynamic_gain );
  }
}

void MC_812::SO( int axis, long output )
{
  COMMAND( "set output " << axis << ", " << output );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> SO( output );
  }
  else
  {
    m_motor[ axis-1 ] -> SO( output );
  }
}

void MC_812::SP( int axis, long deceleration_point )
{
  COMMAND( "set deceleration point " << axis << ", " << deceleration_point );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> SP( deceleration_point );
  }
  else
  {
    m_motor[ axis-1 ] -> SP( deceleration_point );
  }
}

void MC_812::SQ( int axis, long torque )
{
  COMMAND( "set torque " << axis << ", " << torque );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> SQ( torque );
  }
  else
  {
    m_motor[ axis-1 ] -> SQ( torque );
  }
}

void MC_812::SV( int axis, long velocity )
{
  COMMAND( "set velocity " << axis << ", " << velocity );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> SV( velocity );
  }
  else
  {
    m_motor[ axis-1 ] -> SV( velocity );
  }
}

void MC_812::DM( )
{
  COMMAND( "decimal mode" );

  m_mode = dec_mode;
}

void MC_812::HM( )
{
  COMMAND( "hexadecimal mode" );

  m_mode = hex_mode;
}




void MC_812::CS( )
{
  COMMAND( "check sum" );

  AppendOutput( m_ETX );
}

void MC_812::HE( long param )
{
  COMMAND( "help" << param );

  AppendOutput( "help message\x0A\x0D" );
  AppendOutput( m_ETX );
}

void MC_812::TA( long param )
{
  COMMAND( "tell analog " << param );

  AppendOutput( m_ETX );
}

void MC_812::TC( long channel )
{
  COMMAND( "tell channel " << channel );

  AppendOutput( m_ETX );
}

void MC_812::TE( int axis )
{
  COMMAND( "tell error " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> TE( i+1 );
  }
  else
  {
    m_motor[ axis-1 ] -> TE( axis );
  }

  AppendOutput( m_ETX );

  UpdateStatus( );
}

void MC_812::TF( int axis )
{
  COMMAND( "tell following error " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> TF( i+1 );
  }
  else
  {
    m_motor[ axis-1 ] -> TF( axis );
  }
  AppendOutput( m_ETX );
}

void MC_812::TG( int axis )
{
  COMMAND( "tell gain " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> TG( i+1 );
  }
  else
  {
    m_motor[ axis-1 ] -> TG( axis );
  }
  AppendOutput( m_ETX );
}

void MC_812::TO( int axis )
{
  COMMAND( "tell output " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> TO( );
  }
  else
  {
    m_motor[ axis-1 ] -> TO( );
  }
  AppendOutput( m_ETX );
}

void MC_812::TI( )
{
  COMMAND( "tell iterations " );
  AppendOutput( m_ETX );
}

void MC_812::TP( int axis )
{
  COMMAND( "tell position " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> TP( i+1 );
  }
  else
  {
    m_motor[ axis-1 ] -> TP( axis );
  }
  AppendOutput( m_ETX );

  UpdateStatus( );
}

void MC_812::TQ( int axis )
{
  COMMAND( "tell torque " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> TQ( i+1 );
  }
  else
  {
    m_motor[ axis-1 ] -> TQ( axis );
  }
  AppendOutput( m_ETX );
}

void MC_812::TS( int axis )
{
  COMMAND( "tell status " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> TS( axis );
  }
  else
  {
    m_motor[ axis-1 ] -> TS( axis );
  }
  AppendOutput( m_ETX );

  UpdateStatus( );
}

void MC_812::TT( int axis )
{
  COMMAND( "tell target " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> TT( i+1 );
  }
  else
  {
    m_motor[ axis-1 ] -> TT( axis );
  }
  AppendOutput( m_ETX );
}

void MC_812::TV( int axis )
{
  COMMAND( "tell actual velocity " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ )  m_motor[ i ] -> TV( i+1 );
  }
  else
  {
    m_motor[ axis-1 ] -> TV( axis );
  }
  AppendOutput( m_ETX );
}

void MC_812::TY( int axis )
{
  COMMAND( "tell programmed velocity " << axis );

  if ( axis == 0 )
  {
    for( int i = 0; i<motor_count; i++ ) m_motor[ i ] -> TY( i+1 );
  }
  else
  {
    m_motor[ axis-1 ] -> TY( axis );
  }
  AppendOutput( m_ETX );
}

void MC_812::VE( )
{
  COMMAND( "tell version " );

  AppendOutput( "C-812 Simulation\0x0a\0x0d" );
  AppendOutput( "Ver. 1.0\0x0a\0x0d" );
  AppendOutput( m_ETX );
}



void MC_812::StartCmdSeq( )
{
  COMMAND( "start of command sequence" );
}

#ifdef __BCPLUSPLUS__
#pragma argsused
#endif
void MC_812::EndCmdSeq( int length )
{
  //COMMAND("end of command sequence " << length);

  // DOC1, page A17
  //   echos an ETX at the end of all commands,
  //   not merely those which report data
  // tell commands seems to report two trailing ETX
  AppendOutput( m_ETX );
}



void MC_812::Execute( int cmd, int axis, long parameter )
{
  switch ( cmd )
  {
    case CMD_AB:
      AB( axis );
      break;
    case CMD_CP:
      CP( axis );
      break;
    case CMD_FE:
      FE( axis, parameter );
      break;
    case CMD_GH:
      GH( axis );
      break;
    case CMD_GO:
      GO( axis );
      break;
    case CMD_MA:
      MA( axis, parameter );
      break;
    case CMD_MF:
      MF( axis );
      break;
    case CMD_MI:
      MI( parameter );
      break;
    case CMD_MN:
      MN( axis );
      break;
    case CMD_MP:
      MP( parameter );
      break;
    case CMD_MR:
      MR( axis, parameter );
      break;
    case CMD_ST:
      ST( axis );
      break;

    case CMD_BR:
      BR( parameter );
      break;
    case CMD_CF:
      CF( parameter );
      break;
    case CMD_CI:
      CI( parameter );
      break;
    case CMD_CN:
      CN( parameter );
      break;
    case CMD_CT:
      CT( parameter );
      break;
    case CMD_DB:
      DB( axis, parameter );
      break;
    case CMD_DH:
      DH( axis );
      break;
    case CMD_EN:
      EN( );
      break;
    case CMD_EF:
      EF( );
      break;
    case CMD_IA:
      IA( parameter );
      break;
    case CMD_LS:
      LS( axis, parameter );
      break;
    case CMD_RT:
      RT( );
      break;
    case CMD_SA:
      SA( axis, parameter );
      break;
    case CMD_SD:
      SD( axis, parameter );
      break;
    case CMD_SF:
      SF( axis );
      break;
    case CMD_SN:
      SN( axis, parameter );
      break;
    case CMD_SG:
      SG( axis, parameter );
      break;
    case CMD_SE:
      SE( axis, parameter );
      break;
    case CMD_SO:
      SO( axis, parameter );
      break;
    case CMD_SP:
      SP( axis, parameter );
      break;
    case CMD_SQ:
      SQ( axis, parameter );
      break;
    case CMD_SV:
      SV( axis, parameter );
      break;
    case CMD_DM:
      DM( );
      break;
    case CMD_HM:
      HM( );
      break;

    case CMD_CS:
      CS( );
      break;
    case CMD_HE:
      HE( parameter );
      break;
    case CMD_TA:
      TA( parameter );
      break;
    case CMD_TC:
      TC( parameter );
      break;
    case CMD_TE:
      TE( axis );
      break;
    case CMD_TF:
      TF( axis );
      break;
    case CMD_TG:
      TG( axis );
      break;
    case CMD_TO:
      TO( axis );
      break;
    case CMD_TI:
      TI( );
      break;
    case CMD_TP:
      TP( axis );
      break;
    case CMD_TQ:
      TQ( axis );
      break;
    case CMD_TS:
      TS( axis );
      break;
    case CMD_TT:
      TT( axis );
      break;
    case CMD_TV:
      TV( axis );
      break;
    case CMD_TY:
      TY( axis );
      break;
    case CMD_VE:
      VE( );
      break;
    default:
      Error( cmd, axis, parameter );
  }
}


// PRIVATE /////////////////////////////////////////////////////////////////

#define FLAG_OFFSET  0x0800U
#define IN1_OFFSET   0x03fcU
#define IN2_OFFSET   0x03ffU
#define OUT_OFFSET   0x03feU

void MC_812::Put( char* address, char put )
{
  unsigned int addr;
  addr = (unsigned int) address;

  // compare C_812ISA::TC_812ISA( void )
  if ( m_base_addr >= 0xD000 )
  {
    //addr = ( addr - ( 0x10 * ( m_base_addr - 0xD000 ) ) );
    unsigned int addr2 = 0x10 * ( m_base_addr /*- 0xD000*/ ) ;
    addr = addr - addr2;
  }
  else
  {
    //addr = ( addr - ( 0x10 * ( m_base_addr - 0xC000 ) ) );
    unsigned int addr2 = 0x10 * ( m_base_addr /*- 0xC000*/ ) ;
    addr = addr - addr2;
  }

  if ( addr == IN1_OFFSET )
  {
    m_in_char = put;
    m_in_state = true;
  }
  else if ( addr == IN2_OFFSET )
  {
    if ( m_in_state && m_in_char == put )
    {
      m_in[ m_in_pos++ ]  = put;
      // command sequence closed ?
      if ( put == RTN )
      {
        // add trailing \0
        m_in[ m_in_pos ]  = 0;
        // interprete the input
        Interprete( );
        // reset write position
        m_in_pos = 0;
      }
    }
  }
  else if ( 0x010A <= addr && addr <= 0x0119 )
  {
    // direct access to position register
    int motor_id, byte_num;
    motor_id = ( addr - 2 )  & 0x3;
    switch ( addr & 0x1F )
    {
      case 0x0A:
      case 0x0B:
      case 0x0C:
      case 0x0D:
        byte_num = 0;
        break;
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
        byte_num = 1;
        break;
      case 0x12:
      case 0x13:
      case 0x14:
      case 0x15:
        byte_num = 2;
        break;
      case 0x16:
      case 0x17:
      case 0x18:
      case 0x19:
        byte_num = 3;
        break;
    }
    // we expect a writeing direction from the 0th to the 3rd byte ?!
    if ( byte_num == 0 )
    {
      m_motor[ motor_id ] -> CalcPosition( );
      UpdateStatus( );
    }
    long pos = m_motor[ motor_id ] -> m_position;
    char* pp = ( char* ) &pos;
    pp[ byte_num ]  = put;
    m_motor[ motor_id ] -> m_position = pos;

    LOG(  "<cmd-seq>\n<cmd c='C812ISA' type='direct' byte='" << byte_num
          << "'>set position "
          << ( motor_id+1 )
          << ", " << pos << "</cmd>\n</cmd-seq>" << endl );
  }
}

// what should we return if data is requested but
// data available is unset ?
char MC_812::Get( char* address )
{
  unsigned int addr;
  addr = (unsigned int) address;

  // compare C_812ISA::TC_812ISA( void )
  if ( m_base_addr >= 0xD000 )
  {
    unsigned int addr2 = 0x10 * ( m_base_addr /*- 0xD000*/ ) ;
	  addr = addr - addr2;
  }
  else
  {
    unsigned int addr2 = 0x10 * ( m_base_addr /*- 0xC000*/ ) ;
    addr = addr - addr2;
  }

  char ret_val;
  if ( addr == OUT_OFFSET )
  {
    if ( m_out[ m_out_pos ]  != 0 )
    {
      ret_val = m_out[ m_out_pos++ ];
    }
    else
    {
      ret_val = m_ETX;
    }

    // if we reach the terminating 0, we set data available to false
    SetDataAvailable( m_out[ m_out_pos ]  != 0 );

    return ret_val;
  }
  else if ( addr == FLAG_OFFSET )
  {
    // return State
    // set in randomized situations the busy flag
    if ( m_controller_busy_probability > 0 &&
         rand( ) > RAND_MAX * ( 1 - m_controller_busy_probability ) )
      return m_state | 1;
    else
      return m_state;
  }
  else if ( 0x006C <= addr && addr <= 0x007B )
  {
    // direct access to failure register
    int motor_id, byte_num;
    long failure;
    motor_id = ( addr & 0x3 );
    byte_num = ( ( addr >> 2 ) + 1 ) & 0x3;
    // we expect a reading direction from the 0th to the 3rd byte ?!
    // so we calc. the pos. only if the 0th byte is requested
    if ( byte_num == 0 )
    {
      m_motor[ motor_id ] -> CalcPosition( );
      failure = m_motor[ motor_id ] -> m_position - m_motor[ motor_id ] -> m_target;
      LOG( "<cmd-seq>\n<cmd c='C812ISA' type='direct'>tell error "
           << ( motor_id+1 )  << "</cmd>\n"
           << "<out>"
           << failure
           << "</out>\n</cmd-seq>"
           << endl );
      UpdateStatus( );
    }
    else
    {
      failure = m_motor[ motor_id ] -> m_position - m_motor[ motor_id ] -> m_target;
    }

    char* pp = ( char* )  &failure;
    return pp[ byte_num ];
  }
  else if ( 0x010A <= addr && addr <= 0x0119 )
  {
    // direct access to position register
    int motor_id, byte_num;
    motor_id = ( addr - 2 )  & 0x3;
    switch ( addr & 0x1F )
    {
      case 0x0A: case 0x0B: case 0x0C: case 0x0D:
        byte_num = 0;
        break;
      case 0x0E: case 0x0F: case 0x10: case 0x11:
        byte_num = 1;
        break;
      case 0x12: case 0x13: case 0x14: case 0x15:
        byte_num = 2;
        break;
      case 0x16: case 0x17: case 0x18: case 0x19:
        byte_num = 3;
        break;
    }
    // we expect a reading direction from the 0th to the 3rd byte ?!
    // so we calc. the pos. only if the 0th byte is requested
    if ( byte_num == 0 )
    {
      m_motor[ motor_id ] -> CalcPosition( );
      LOG(  "<cmd-seq>\n<cmd c='C812ISA' type='direct'>tell position "
            << ( motor_id+1 ) << "</cmd>\n"
            << "<out>"
            << m_motor[ motor_id ] -> m_position
            << "</out>\n</cmd-seq>"
            << endl );
      UpdateStatus( );
    }
    long pos = m_motor[ motor_id ] -> m_position;
    char* pp = ( char* ) &pos;
    return pp[ byte_num ];
  }
  else
  {
    return 0;    //??
  }

}


void MC_812::Interprete( )
{
  ResetOutput( );

  LOG(  "<cmd-seq>" << endl );
  ::parse_command_string( m_in, this, m_mode );

  char* pc = m_out;
  LOG(  "<out>" );
  while( *pc )
  {
    if ( *pc == m_ETX )
    {
      LOG( "{ETX}" );
    }
    else if ( *pc == CR )
    {
      LOG( "{CR}" );
    }
    else if ( *pc == LF )
    {
      LOG( "{LF}\n" );
    }
    else
    {
      LOG( *pc );
    }
    pc++;
  }
  LOG( "</out>\n" );

  SetDataAvailable( true );
  LOG( "</cmd-seq>" << endl );
}

void MC_812::Interprete( char* cmds )
{
  ResetOutput( );

  parse_command_string( cmds, this, m_mode );
  SetDataAvailable( m_out[ 0 ]  != 0 );

  LOG( "output: " << m_out << endl );
}


void MC_812::ResetOutput( )
{
  m_out[ 0 ]  = 0;
  m_out_pos = 0;

  SetDataAvailable( false );
}

void MC_812::AppendOutput( char* s )
{
  strcat( m_out, s );
}

void MC_812::AppendOutput( char c )
{
  char*cp;
  for( cp = m_out; *cp != 0; cp++ );
  *cp = c;
  cp++;
  *cp = 0;
}


/////// Motors /////////////////////////////////////////

MC_812::Motor::Motor( MC_812* controller )  : m_Controller( controller )
{
  // * for some default values see DOC1, page A23
  m_limit = 4096;               // *
  m_acceleration = 200000L;     // *
  m_deceleration = 200000L;     // *
  m_gain = 5;                   // in cmd descr. 32 ??
  m_dynamic_gain = 5;           // in cmd descr. 32 ??
  m_output = 0;                 // DEFAULT?
  m_deceleration_point = 25912; // *
  m_torque    = 127;            // *
  m_velocity_max = 100000L;     // *
  m_dead_band    = 0;           // DEFAULT?

  m_target = 0;
  m_position = 0;
  m_position_time = 0;
  m_velocity = 0;

  m_physical_position_range = LONG_MAX;
  m_physical_position = LONG_MAX / 2;

  m_hysteresis = 0;
  m_last_direction = true;

  m_status = 0;
  Set_OnTargetFlag( true );
  Set_LimitFlag( false );
  Set_MotorOffFlag( true );
  Set_SlewFlag( false );
  Set_CommandErrorFlag( false );
  Set_SlaveFlag( false );
  Set_FindEdgeFlag( false );
  Set_WaitingFlag( false );
}


void MC_812::Motor::AB( )
{
  m_target = CalcPosition( );
}

void MC_812::Motor::CP( )
{
}

#ifdef __BCPLUSPLUS__
#pragma argsused
#endif
void MC_812::Motor::FE( long direction )
{
}

void MC_812::Motor::GH( )
{
  // "equivaliant to a MA0 command"
  MA( 0 );
}

void MC_812::Motor::GO( )
{
  // "functionally equivalent to MN"
  MN( );
}

void MC_812::Motor::MA( long position )
{
  //if (! between(position, -4294976296L, 4294976296L) )
  if (! between(position, LONG_MIN, LONG_MAX) )
  {
    // this can't really happen, this test should be at an other place
    LOGERROR_("MA: position outside valid vaue range");
  }

  if ( m_last_direction == false && position > m_position )
  {
    // swap direction from <== to ==>
    m_last_direction = true;
    m_physical_position -= m_hysteresis;
  }
  else if ( m_last_direction == true && position < m_position )
  {
    // swap direction from ==> to <==
    m_last_direction = false;
    m_physical_position += m_hysteresis;
  }

  CalcPosition( );
  Set_LimitFlag( false );
  m_target = position;
}

void MC_812::Motor::MF( )
{
  CalcPosition( );
  Set_MotorOffFlag( true );
}

void MC_812::Motor::MN( )
{
  Set_MotorOffFlag( false );
  m_position_time = m_Controller -> GetCurrentTime( );
}

void MC_812::Motor::MR( long distance )
{
  if ( m_last_direction == false && distance > 0 )
  {
    // swap direction from <== to ==>
    m_last_direction = true;
    m_physical_position -= m_hysteresis;
  }
  else if ( m_last_direction == true && distance < 0 )
  {
    // swap direction from ==> to <==
    m_last_direction = false;
    m_physical_position += m_hysteresis;
  }

  CalcPosition( );
  Set_LimitFlag( false );
  m_target += distance;
}

void MC_812::Motor::ST( )
{
  // "funcionally equivalent to MF"
  MF( );
}


void MC_812::Motor::DB( long width )
{
  if (! between(width, 1L, 126L) )
  {
    LOGERROR_("DB: width outside valid value range [1..126]");
  }
  m_dead_band = width;
}

void MC_812::Motor::DH( )
{
  m_position = 0;
  m_position_time = m_Controller -> GetCurrentTime( );

  m_target = 0;
}


void MC_812::Motor::LS( long limit )
{
  m_limit = limit;
}

void MC_812::Motor::SA( long acceleration )
{
  if (! between(acceleration, 0L, 8388608L) )
  {
    LOGERROR_("SA: acceleration outside valid value range [0..8388608]");
  }
  m_acceleration = acceleration;
}

void MC_812::Motor::SD( long deceleration )
{
  if (! between(deceleration, 0L, 8388608L) )
  {
    LOGERROR_("SD: deceleration outside valid value range [0..8388608]");
  }
  m_deceleration = deceleration;
}

void MC_812::Motor::SG( long gain )
{
  if (! between(gain, 1L, 254L) )
  {
    LOGERROR_("SG: gain outside valid value range [1..254]");
  }
  m_gain = gain;
}

void MC_812::Motor::SE( long dynamic_gain )
{
  if (! between(dynamic_gain, 1L, 254L) )
  {
    LOGERROR_("SE: dynamic gain outside valid value range [1..254]");
  }
  m_dynamic_gain = dynamic_gain;
}

void MC_812::Motor::SO( long output )
{
  if (! between(output, 0L, 255L) )
  {
    LOGERROR_("SO: output outside valid value range [0..255]");
  }
  m_output = output;
}

void MC_812::Motor::SP( long deceleration_point )
{
  if (! between(deceleration_point, 0L, 65535L) )
  {
    LOGERROR_("SP: deceleration point outside valid value range [0..65535]");
  }
  m_deceleration_point = deceleration_point;
}

void MC_812::Motor::SQ( long torque )
{
  if (! between(torque, 1L, 126L) )
  {
    LOGERROR_("SQ: torque outside valid value range [1..126]");
  }
  m_torque = torque;
}

void MC_812::Motor::SV( long velocity )
{
  if (! between(velocity, 0L, 8388608L) )
  {
    LOGERROR_("SV: velocity outside valid value range [0..8388608]");
  }
  m_velocity_max = velocity;
}


void MC_812::Motor::TE( int axis )
{
  char buf[ 16+5 ]; // 2 + 1 + 10 + 2 + 1

  CalcPosition( );

  // diferent string length for pos/neg values
  //   01E-0000001234
  //   01E0000001234
  if ( ( m_position - m_target )  < 0 )
    if ( m_Controller -> m_mode == dec_mode )
    {
      sprintf( buf, "%02iE-%010li\x0D\x0A", axis, m_target - m_position );
    }
    else
    {
      sprintf( buf, "%02iE-%010lX\x0D\x0A", axis, m_target - m_position );
    }
  else
    if ( m_Controller -> m_mode == dec_mode )
    {
      sprintf( buf, "%02iE%010li\x0D\x0A", axis, m_position - m_target );
    }
    else
    {
      sprintf( buf, "%02iE%010lX\x0D\x0A", axis, m_position - m_target );
    }

  m_Controller -> AppendOutput( buf );
}

void MC_812::Motor::TF( int axis )
{
  // currently always 0
  // because in our simulation the motor moves always
  // in the programmed speed

  char buf[ 16+5 ]; // 2 + 1 + 10 + 2 + 1

  if ( m_Controller -> m_mode == dec_mode )
  {
    sprintf( buf, "%02iF%010li\x0D\x0A", axis, 0 );
  }
  else
  {
    sprintf( buf, "%02iF%010lX\x0D\x0A", axis, 0 );
  }

  m_Controller -> AppendOutput( buf );
}

void MC_812::Motor::TG( int axis )
{
  char buf[ 10+5 ]; // 2 + 1 + 4 + 2 + 1

  if ( m_Controller -> m_mode == dec_mode )
  {
    sprintf( buf, "%02iG%04li\x0D\x0A", axis, m_gain );
  }
  else
  {
    sprintf( buf, "%02iG%04lX\x0D\x0A", axis, m_gain );
  }

  m_Controller -> AppendOutput( buf );
}

void MC_812::Motor::TO( )
{}


void MC_812::Motor::TP( int axis )
{
  char buf[ 16+5 ]; // 2 + 1 + 10 + 2 + 1

  CalcPosition( );
  // different string length for pos/neg values
  //   01P0000001234
  //   01P-0000001234
  if ( m_position >= 0 )
    if ( m_Controller -> m_mode == dec_mode )
    {
      sprintf( buf, "%02iP%010li\x0D\x0A", axis, m_position );
    }
    else
    {
      sprintf( buf, "%02iP%010lX\x0D\x0A", axis, m_position );
    }
  else
    if ( m_Controller -> m_mode == dec_mode )
    {
      sprintf( buf, "%02iP-%010li\x0D\x0A", axis, -m_position );
    }
    else
    {
      sprintf( buf, "%02iP-%010lX\x0D\x0A", axis, -m_position );
    }

  m_Controller -> AppendOutput( buf );
}

void MC_812::Motor::TQ( int axis )
{
  char buf[ 10+5 ]; // 2 + 1 + 4 + 2 + 1

  if ( m_Controller -> m_mode == dec_mode )
  {
    sprintf( buf, "%02iQ%04li\x0D\x0A", axis, m_torque );
  }
  else
  {
    sprintf( buf, "%02iQ%04lX\x0D\x0A", axis, m_torque );
  }

  m_Controller -> AppendOutput( buf );
}

void MC_812::Motor::TS( int axis )
{
  char buf[ 10+5 ]; // 2 + 1 + 4 + 2 + 1

  CalcPosition( );

  if ( m_position == m_target &&
       m_Controller -> GetCurrentTime( )  - m_position_time > 128 )
  {
    Set_WaitingFlag( true );
  }

  if ( m_Controller -> m_mode == dec_mode )
  {
    sprintf( buf, "%02iS%04i\x0D\x0A", axis, ( int ) m_status );
  }
  else
  {
    sprintf( buf, "%02iS%04X\x0D\x0A", axis, ( int ) m_status );
  }

  m_Controller -> AppendOutput( buf );
}

void MC_812::Motor::TT( int axis )
{
  char buf[ 16+5 ]; // 2 + 1 + 10 + 2 + 1

  if ( m_Controller -> m_mode == dec_mode )
  {
    sprintf( buf, "%02iT%010li\x0D\x0A", axis, m_target );
  }
  else
  {
    sprintf( buf, "%02iT%010lX\x0D\x0A", axis, m_target );
  }

  m_Controller -> AppendOutput( buf );
}

void MC_812::Motor::TV( int axis )
{
  char buf[ 16+5 ]; // 2 + 1 + 10 + 2 + 1

  CalcPosition( );

  if ( m_Controller -> m_mode == dec_mode )
  {
    sprintf( buf, "%02iV%010li\x0D\x0A", axis, m_velocity );
  }
  else
  {
    sprintf( buf, "%02iV%010lX\x0D\x0A", axis, m_velocity );
  }

  m_Controller -> AppendOutput( buf );
}

void MC_812::Motor::TY( int axis )
{
  char buf[ 16+5 ]; // 2 + 1 + 10 + 2 + 1

  if ( m_Controller -> m_mode == dec_mode )
  {
    sprintf( buf, "%02iY%010li\x0D\x0A", axis, m_velocity_max );
  }
  else
  {
    sprintf( buf, "%02iY%010lX\x0D\x0A", axis, m_velocity_max );
  }

  m_Controller -> AppendOutput( buf );
}




// position with constant max velocity, to fast

long MC_812::Motor::CalcPosition_MaxVelocity( )
{
  long cur_time = m_Controller -> GetCurrentTime( );
  long dif_time = cur_time - m_position_time;

  if ( !IsMotorOff( )  && ( m_target != m_position ) )
  {
    if ( m_target < m_position )
    {
      // move in "left" direction
      m_position -= ( m_velocity_max * dif_time )  / 1000;
      // over shot
      if ( m_position < m_target )
      {
        m_position = m_target;
      }
    }
    else
    {
      // move in "right" direction
      m_position += ( m_velocity_max * dif_time )  / 1000;
      // over shot
      if ( m_position > m_target )
      {
        m_position = m_target;
      }
    }
  }

  m_position_time = cur_time;
  return m_position;
}

#define _sq(x)        ((x) * (x))
#define _error        (m_target - m_position + \
                      ((m_target>=m_position) ? (long)-ceil(s_1) : (long)ceil(s_1)))
#define _error_       (m_target - m_position)
#define _decel_dist   (_sq(v_1) / (2 * m_deceleration))
#define _decel_dist_  (_sq(m_velocity) / (double)(2 * m_deceleration))

//
//   ^
// v |
//   |           ______________
//   |         /|              |\
//   |       /  |              | \
//   |     /    |              |  \
//   |   /      |              |   \
//   | /        |              |    \
//   +----------|--------------|-----|------------>
//     phase 1  |  phase 2     | phase 3        t
//


long MC_812::Motor::CalcPosition_Ideal( )
{
  long t = m_Controller -> GetCurrentTime( );
  double delta_t = ( t - m_position_time ) / 1000.0;
  double s_1, s_2, v_1, v_2;

  // check that we are really moving
  if ( IsMotorOff( ) )  { goto end; }
  if ( m_target == m_position )  { Set_OnTargetFlag( true ); goto end; }
  if ( ! ( delta_t > 0.0 ) )  goto end;

  Set_OnTargetFlag( false );
  Set_WaitingFlag( false );

  if ( labs( _error_ ) > _decel_dist_ )
  {
    if ( m_velocity < m_velocity_max )
    {
      // in phase 1
      s_1 = ( m_acceleration / 2 ) * _sq(delta_t) +
              m_velocity * delta_t;
      v_1 = m_acceleration * delta_t + m_velocity;

      if ( labs(_error) > _decel_dist )
      {
        if ( v_1 < m_velocity_max )
        {
          // stay in phase 1
          s_2 = s_1;
          v_2 = v_1;
        }
        else
        {
          // change to phase 2
          s_2 = m_velocity_max * delta_t;
          s_2 = fabs( s_1 + s_2 )  / 2;
          v_2 = m_velocity_max;
        }
        goto check_target_reached;
      }
      else
      {
        goto change_to_phase_3;
      }
    }
    else
    {
      // in Phase 2
      s_1 = m_velocity_max * delta_t;
      v_1 = m_velocity_max;

      if ( labs( _error )  > _decel_dist )
      {
        // stay in phase 2
        s_2 = s_1;
        v_2 = m_velocity_max;
        goto check_target_reached;
      }
      else
      {
        goto change_to_phase_3;
      }
    }
  }
  else
  {
    // in phase 3
    s_2 = ( -m_deceleration / 2 ) * _sq( delta_t )  +
          m_velocity * delta_t;
    v_2 = ( -m_deceleration * delta_t )  + m_velocity;
    goto check_target_reached;
  }


change_to_phase_3:
  {
    s_2 = ( -m_deceleration / 2 )  * _sq( delta_t )  +
          m_velocity * delta_t;
    s_2 = fabs( s_1 + s_2 )  / 2;
    v_2 = ( -m_deceleration * delta_t )  + m_velocity;
    v_2 = fabs( v_1 + v_2 )  / 2;
  }

check_target_reached:
  {
    if ( m_position <= m_target )
    {
      // move direction -->
      m_position += (long)ceil( s_2 );
      m_physical_position += (long)ceil( s_2 );
      if ( m_physical_position > m_physical_position_range )
      {
        // hardware limit switch reached
        Set_LimitFlag( true );
        // stop (somewhat suddenly)
        m_velocity = 0;
        // back to limit switch
        m_position -= m_physical_position - m_physical_position_range;
        m_physical_position = m_physical_position_range;
        // set new target to remove from limit switch
        m_target = m_position - m_limit;
        // apply hysteresis because of reverting direction
        m_physical_position += m_hysteresis;
        m_last_direction = false;
      }
      else
      {
        m_velocity = (m_velocity < v_2) ? (long)ceil( v_2 ) : (long)floor( v_2 );
        if ( ! ( m_position < m_target && v_2 > 0.0 )  )
        {
          // target reached or over shot
          m_physical_position -= ( m_position - m_target );
          m_position = m_target;
          // stop
          m_velocity = 0;
        }
      }
    }
    else
    {
      // move direction <--
      m_position -= (long)ceil( s_2 );
      m_physical_position -= (long)ceil( s_2 );
      if ( m_physical_position < 0 )
      {
        // hardware limit switch reached
        Set_LimitFlag( true );
        // stop (somewhat suddenly)
        m_velocity = 0;
        // back to the limit switch
        m_position += labs( m_physical_position );
        m_physical_position = 0;
        // set new target to remove from limit switch
        m_target = m_position + m_limit;
        // apply hysteresis because of reverting direction
        m_physical_position -= m_hysteresis;
        m_last_direction = true;
      }
      else
      {
        m_velocity = (m_velocity < v_2) ? (long)ceil( v_2 ) : (long)floor( v_2 );
        if ( ! ( m_position > m_target && v_2 > 0.0 )  )
        {
          // target reached or over shot
          m_physical_position += ( m_target - m_position );
          m_position = m_target;
          // stop
          m_velocity = 0;
        }
      }
    }
  }

  if ( m_Controller->m_log_stream )
  {
    ( *m_Controller->m_log_stream ) << setbase( 10 )
          << "<info c='c812'>time " << t << " position " << m_position
          << "/" << m_physical_position
          << " velocity " << m_velocity << "</info>" << endl;
  }

end:
  {
    if ( delta_t > 0.0 )  m_position_time = t;

    return m_position;
  }
}


void MC_812::UpdateStatusText( )
{
  char tmp[ 100 ];
  int i;

  m_status_text[ 0 ]  = 0;
  for( i=0; i<4; i++ )
  {
    sprintf( tmp, "%cM%i: %li, %li, %li\r\n   [%i%i%i%i%i%i%i%i]\r\n",
             ( m_motor[ i ] -> m_last_direction ? '>' : '<' ),
             i+1,
             m_motor[ i ] -> m_position,
             m_motor[ i ] -> m_physical_position,
             m_motor[ i ] -> m_target - m_motor[ i ] ->m_position,
             ( m_motor[ i ] -> m_status & 128 ) == 128,
             ( m_motor[ i ] -> m_status & 64 ) == 64,
             ( m_motor[ i ] -> m_status & 32 ) == 32,
             ( m_motor[ i ] -> m_status & 16 ) == 16,
             ( m_motor[ i ] -> m_status & 8 ) == 8,
             ( m_motor[ i ] -> m_status & 4 ) == 4,
             ( m_motor[ i ] -> m_status & 2 ) == 2,
             ( m_motor[ i ] -> m_status & 1 ) == 1
           );
    strcat( m_status_text, tmp );
  }
};

const char* MC_812::GetStatusText( )
{
  int i;
  for( i=0; i<4; i++ )
  {
    m_motor[ i ] -> CalcPosition( );
  }
  UpdateStatusText( );
  return m_status_text;
}

