
#ifndef MC_812_H
#define MC_812_H

#ifdef __BCPLUSPLUS__
#define bool int
#define true 1
#define false 0
#endif

#include <assert.h>
#include <iostream.h>
#include <windows.h>
#include <mmsystem.h>
#include "../export.h"


class MSIMAPI MC_812
{
  public:
    MC_812( );
    ~MC_812( );

    ////////////////////////////////////////////////////
    // simulation control
    void SetControllerBusyProbability( double value );
    double GetControllerBusyProbability( );

    void SetPhysicalPosition( int board_id, long value );
    long GetPhysicalPosition( int board_id );
    void SetPhysicalPositionRange( int board_id, long value );
    long GetPhysicalPositionRange( int board_id );
    void SetHysteresis( int board_id, long value );
    long GetHysteresis( int board_id );
    void SetLastDirection( int board_id, bool direction );
    bool GetLastDirection( int board_id );
    void SetBaseAddr( unsigned );
    unsigned GetBaseAddr( );

    void SetLogStream( ostream* );
    ostream* GetLogStream( );
    void SetLogLevel( int );
    int  GetLogLevel( );
    const char* GetStatusText( );
  private:
    void UpdateStatus( );
    void UpdateStatusText( );


    ////////////////////////////////////////////////////
  private:
    // starts interpretation of the input buffer
    // calling the parser
    void Interprete( );
    // same as Interprete(void), buy interpretes the given string
    // for testing purposes
    void Interprete( char* cmds );

  public:
    // methods for for use by parser (thats why they must be public)
    // -- called for every command found in the parsed string
    void Execute( int cmd, int axis, long parameter );
    // -- called if the parser finds an error in the parsed string
    void Error( int cmd, int axis, long parameter );
    // -- called at the begin of the parsed string (currently not used)
    void StartCmdSeq( );
    // -- called it the end of the parsed string is found
    void EndCmdSeq( int length );



  private:
    ////////////////////////////////////////////////////
    // one member function for each command
    void AB( int axis );
    void CP( int axis );
    void FE( int axis, long direction );
    void GH( int axis );
    void GO( int axis );
    void MA( int axis, long position );
    void MF( int axis );
    void MI( long point );
    void MN( int axis );
    void MP( long point );
    void MR( int axis, long distance );
    void ST( int axis );

    void BR( long rate );
    void CF( long channel );
    void CI( long channel );
    void CN( long channel );
    void CT( long channel );
    void DB( int axis, long width );
    void DH( int axis );
    void EN( );
    void EF( );
    void IA( long address );
    void LS( int axis, long limit );
    void RT( );
    void SA( int axis, long acceleration );
    void SD( int axis, long deceleration );
    void SF( int axis );
    void SN( int axis, long master );
    void SG( int axis, long gain );
    void SE( int axis, long dynamic_gain );
    void SO( int axis, long output );
    void SP( int axis, long deceleration_point );
    void SQ( int axis, long torque );
    void SV( int axis, long velocity );
    void DM( );
    void HM( );

    void CS( );
    void HE( long param );
    void TA( long param );
    void TC( long channel );
    void TE( int axis );
    void TF( int axis );
    void TG( int axis );
    void TO( int axis );
    void TI( );
    void TP( int axis );
    void TQ( int axis );
    void TS( int axis );
    void TT( int axis );
    void TV( int axis );
    void TY( int axis );
    void VE( );

    /*
    void EM(long macro);
    void MC(long macro);
    void MD(long macro);
    void RM();
    void RP(long count);
    void TM(long macro);
    void WA(long time);
    void WD(int axis, long delta);
    void WE(int axis,
    */

  public:
    ////////////////////////////////////////////////////
    // main input/output functions
    void Put( char* address, char put );
    char Get( char* addr );


  private:

    unsigned m_base_addr;

    // contains the last (to IN?_OFFSET) putted char
    char  m_in_char;
    // true if the last put was to IN1_OFFSET, otherwise false
    char  m_in_state;
    // input buffer, stores the collected cmd seq
    char  m_in[ 1024 ];
    // pos. for next writeing operation
    int   m_in_pos;
    // output buffer, stores the complete report of a cmd seq
    char  m_out[ 1024 ];
    // pos. of the next byte to be read
    int   m_out_pos;
    // status byte
    char  m_state;
    // current ETX char
    char  m_ETX;

    // appends a string to to output buffer
    void AppendOutput( char* );
    // appends a char to to output buffer
    void AppendOutput( char );

    // resets the output buffer
    void ResetOutput( );
    // input/output mode (how to report result)
    enum { dec_mode = 0, hex_mode = 1 } m_mode;

    bool m_echo;

    // HELPER /////
    // setting or testing some status bits 
    void SetDataAvailable( bool );
    bool IsDataAvailable( );
    void SetBusy( bool );
    bool IsBusy( );


    double m_controller_busy_probability;


  private:
    class Motor
    {
      public:
        Motor( MC_812* controller );

        void AB( );
        void CP( );
        void FE( long direction );
        void GH( );
        void GO( );
        void MA( long position );
        void MF( );
        void MN( );
        void MR( long distance );
        void ST( );

        void DB( long width );
        void DH( );

        void LS( long limit );
        void SA( long acceleration );
        void SD( long deceleration );
        void SG( long gain );
        void SE( long dynamic_gain );
        void SO( long output );
        void SP( long deceleration_point );
        void SQ( long torque );
        void SV( long velocity );

        void TE( int axis );
        void TF( int axis );
        void TG( int axis );
        void TO( );
        void TP( int axis );
        void TQ( int axis );
        void TS( int axis );
        void TT( int axis );
        void TV( int axis );
        void TY( int axis );

      private:
        MC_812* m_Controller;

        long m_limit;
        long m_dead_band;

        long m_gain;
        long m_dynamic_gain;
        long m_output;
        long m_torque;

        long m_acceleration;
        long m_deceleration;
        long m_deceleration_point;
        // programmed; in encoder counts per second
        long m_velocity_max;
        // actual at m_position_time
        long m_velocity;

        long m_target;
        // position at m_position_time
        long m_position;
        long m_position_time;

        long m_physical_position_range;
        long m_physical_position;

        long m_hysteresis;
        // true == right (increasing pos.), false == left
        bool m_last_direction;

        long CalcPosition( );
        long CalcPosition_MaxVelocity( );
        long CalcPosition_Ideal( );

        /////////////////////////////////////////////////////////
        // status
        char m_status;
        void Set_OnTargetFlag( bool on );
        void Set_LimitFlag( bool );
        void Set_MotorOffFlag( bool );
        void Set_SlewFlag( bool );
        void Set_CommandErrorFlag( bool );
        void Set_SlaveFlag( bool );
        void Set_FindEdgeFlag( bool );
        void Set_WaitingFlag( bool );

        bool IsMotorOff( );

        friend class MC_812;
    };
    friend class Motor;

    enum { motor_count = 4 };
    Motor* m_motor[ motor_count ];

    ostream* m_log_stream;
    int m_log_level;

    // returns the time in milliseconds since programm start
    long GetCurrentTime( );
    long m_starttime;

    char m_status_text[ 1024 ];

};


extern MC_812*  theMC_812;
#ifdef __BCPLUSPLUS__
  MC_812* MSIMAPI getTheC812();
#else
  MSIMAPI MC_812* getTheC812();
#endif


inline void MC_812::SetDataAvailable( bool on )
{
  // bit 1 in m_State is the data available flag
  // set (1) if there is a valid char
  if ( on )
    m_state |= 2;
  else
    m_state &= ~2;
}

inline bool MC_812::IsDataAvailable( )
{
  return ( (m_state & 2) == 2 )  ? true : false;
}

inline void MC_812::SetBusy( bool on )
{
  // bit 0 in m_state is the busy flag
  // set (1) if the controller is busy
  if ( on )
    m_state |= 1;
  else
    m_state &= ~1;
}

inline bool MC_812::IsBusy( )
{
  return ( (m_state & 1) == 1 ) ? true : false;
}

inline
long MC_812::GetCurrentTime( )
{
  return ( timeGetTime() - m_starttime );
}

inline
void MC_812::SetControllerBusyProbability( double value )
{
  assert( 0 <= value && value <= 1 );
  m_controller_busy_probability = value;
  UpdateStatus( );
}

inline
double MC_812::GetControllerBusyProbability( )
{
  return m_controller_busy_probability;
}

inline
void MC_812::SetPhysicalPositionRange( int board_id, long value )
{
  assert( 1 <= board_id && board_id <= 4 );
  m_motor[ board_id - 1 ]  -> m_physical_position_range = value;
  UpdateStatus( );
}

inline
long MC_812::GetPhysicalPositionRange( int board_id )
{
  assert( 1 <= board_id && board_id <= 4 );
  return m_motor[ board_id - 1 ]  -> m_physical_position_range;
}

inline
void MC_812::SetPhysicalPosition( int board_id, long value )
{
  assert( 1 <= board_id && board_id <= 4 );
  m_motor[ board_id - 1 ]  -> m_physical_position = value;
  UpdateStatus( );
}

inline
long MC_812::GetPhysicalPosition( int board_id )
{
  assert( 1 <= board_id && board_id <= 4 );
  return m_motor[ board_id - 1 ]  -> m_physical_position;
}

inline
void MC_812::SetHysteresis( int board_id, long value )
{
  assert( 1 <= board_id && board_id <= 4 );
  m_motor[ board_id - 1 ]  -> m_hysteresis = value;
}

inline
long MC_812::GetHysteresis( int board_id )
{
  assert( 1 <= board_id && board_id <= 4 );
  return m_motor[ board_id - 1 ]  -> m_hysteresis;
}

inline
void MC_812::SetLastDirection( int board_id, bool direction )
{
  assert( 1 <= board_id && board_id <= 4 );
  m_motor[ board_id - 1 ]  -> m_last_direction = direction;
}
inline
bool MC_812::GetLastDirection( int board_id )
{
  assert( 1 <= board_id && board_id <= 4 );
  return m_motor[ board_id - 1 ]  -> m_last_direction;
}

inline
void MC_812::SetBaseAddr( unsigned addr )
{
  m_base_addr = addr;
}

inline
unsigned MC_812::GetBaseAddr( )
{
  return m_base_addr;
}

inline
void MC_812::SetLogStream( ostream* os )
{
  m_log_stream = os;
}

inline
ostream* MC_812::GetLogStream( )
{
  return m_log_stream;
}

inline
void MC_812::SetLogLevel( int lvl )
{
  m_log_level = lvl;
}

inline
int MC_812::GetLogLevel( )
{
  return m_log_level;
}


inline
void MC_812::Motor::Set_OnTargetFlag( bool on )
{
  // bit 0
  if ( on ) 	m_status |= 1;
  else	m_status &= ~1;
}

inline
void MC_812::Motor::Set_LimitFlag( bool on )
{
  // bit 1
  if ( on ) 	m_status |= 2;
  else	m_status &= ~2;
}

inline
void MC_812::Motor::Set_MotorOffFlag( bool on )
{
  // bit 2
  if ( on ) 	m_status |= 4;
  else	m_status &= ~4;
}

inline
void MC_812::Motor::Set_SlewFlag( bool on )
{
  // bit 3
  if ( on ) 	m_status |= 8;
  else	m_status &= ~8;
}

inline
void MC_812::Motor::Set_CommandErrorFlag( bool on )
{
  // bit 4
  if ( on ) 	m_status |= 16;
  else	m_status &= ~16;
}

inline
void MC_812::Motor::Set_SlaveFlag( bool on )
{
  // bit 5
  if ( on ) 	m_status |= 32;
  else	m_status &= ~32;
}

inline
void MC_812::Motor::Set_FindEdgeFlag( bool on )
{
  // bit 6
  if ( on ) 	m_status |= 64;
  else	m_status &= ~64;
}

inline
void MC_812::Motor::Set_WaitingFlag( bool on )
{
  // bit 7
  if ( on ) 	m_status |= 128;
  else	m_status &= ~128;
}

inline
bool MC_812::Motor::IsMotorOff( )
{
  return ( ( m_status & 4 ) == 4 )  ? true : false;
}

inline
long MC_812::Motor::CalcPosition( )
{
  //return CalcPosition_MaxVelcity();
  return CalcPosition_Ideal( );
}

#endif

