
%option noyywrap
%option never-interactive

%{
#include "parsery.h"
#include "cmd.h"
#include <string.h>
#include <stdlib.h>

#define YY_DECL int yylex( char* _input_buffer, int _input_size, int* _read_pos, int _mode )
static char* input_buffer;
static int   input_size;
static int*  read_pos;

#define YY_INPUT(buf,result,max_size) \
 { \
	if (*read_pos >= input_size) { \
		result = YY_NULL; \
	} else {\
		strncpy(buf, input_buffer + *read_pos, max_size); \
		result = ((max_size <= (input_size - *read_pos)) ? max_size : (input_size - *read_pos)); \
		*read_pos += result; \
	}; \
 };


%}

%start DECMODE HEXMODE

COMMAND [a-zA-Z]{2,2}
ENDCHAR [,\r]
A [Aa]
B [Bb]
C [Cc]
D [Dd]
E [Ee]
F [Ff]
G [Gg]
H [Hh]
I [Ii]
K [Kk]
L [Ll]
M [Mm]
N [Nn]
O [Oo]
P [Pp]
Q [Qq]
R [Rr]
S [Ss]
T [Tt]
U [Uu]
V [Vv]
W [Ww]
X [Xx]
Y [Yy]
Z [Zz]




%%
 input_buffer = _input_buffer;
 input_size = _input_size;
 read_pos = _read_pos;
 if (_mode != 0) BEGIN HEXMODE;



[1-4]/{COMMAND} {
	yylval.int_val = yytext[0] - 48;
	return(AXIS);
}

{A}{B} { yylval.int_val = CMD_AB; return(CMD); }
{C}{P} { yylval.int_val = CMD_CP; return(CMD); }
{F}{E} { yylval.int_val = CMD_FE; return(CMD); }
{G}{H} { yylval.int_val = CMD_GH; return(CMD); }
{G}{O} { yylval.int_val = CMD_GO; return(CMD); }
{M}{A} { yylval.int_val = CMD_MA; return(CMD); }
{M}{F} { yylval.int_val = CMD_MF; return(CMD); }
{M}{I} { yylval.int_val = CMD_MI; return(CMD); }
{M}{N} { yylval.int_val = CMD_MN; return(CMD); }
{M}{P} { yylval.int_val = CMD_MP; return(CMD); }
{M}{R} { yylval.int_val = CMD_MR; return(CMD); }
{S}{T} { yylval.int_val = CMD_ST; return(CMD); }


{B}{R} { yylval.int_val = CMD_BR; return(CMD); }
{C}{F} { yylval.int_val = CMD_CF; return(CMD); }
{C}{I} { yylval.int_val = CMD_CI; return(CMD); }
{C}{N} { yylval.int_val = CMD_CN; return(CMD); }
{C}{T} { yylval.int_val = CMD_CT; return(CMD); }
{D}{B} { yylval.int_val = CMD_DB; return(CMD); }
{D}{H} { yylval.int_val = CMD_DH; return(CMD); }
{E}{F} { yylval.int_val = CMD_EF; return(CMD); }
{E}{N} { yylval.int_val = CMD_EN; return(CMD); }
{I}{A} { yylval.int_val = CMD_IA; return(CMD); }
{L}{S} { yylval.int_val = CMD_LS; return(CMD); }
{R}{T} { yylval.int_val = CMD_RT; return(CMD); }
{S}{A} { yylval.int_val = CMD_SA; return(CMD); }
{S}{D} { yylval.int_val = CMD_SD; return(CMD); }
{S}{E} { yylval.int_val = CMD_SE; return(CMD); }
{S}{F} { yylval.int_val = CMD_SF; return(CMD); }
{S}{G} { yylval.int_val = CMD_SG; return(CMD); }
{S}{N} { yylval.int_val = CMD_SN; return(CMD); }
{S}{O} { yylval.int_val = CMD_SO; return(CMD); }
{S}{P} { yylval.int_val = CMD_SP; return(CMD); }
{S}{Q} { yylval.int_val = CMD_SQ; return(CMD); }
{S}{V} { yylval.int_val = CMD_SV; return(CMD); }

{C}{S} { yylval.int_val = CMD_CS; return(CMD); }
{H}{E} { yylval.int_val = CMD_HE; return(CMD); }
{T}{A} { yylval.int_val = CMD_TA; return(CMD); }
{T}{B} { yylval.int_val = CMD_TB; return(CMD); }
{T}{C} { yylval.int_val = CMD_TC; return(CMD); }
{T}{D} { yylval.int_val = CMD_TD; return(CMD); }
{T}{E} { yylval.int_val = CMD_TE; return(CMD); }
{T}{F} { yylval.int_val = CMD_TF; return(CMD); }
{T}{G} { yylval.int_val = CMD_TG; return(CMD); }
{T}{I} { yylval.int_val = CMD_TI; return(CMD); }
{T}{L} { yylval.int_val = CMD_TL; return(CMD); }
{T}{N} { yylval.int_val = CMD_TN; return(CMD); }
{T}{O} { yylval.int_val = CMD_TO; return(CMD); }
{T}{P} { yylval.int_val = CMD_TP; return(CMD); }
{T}{Q} { yylval.int_val = CMD_TQ; return(CMD); }
{T}{R} { yylval.int_val = CMD_TR; return(CMD); }
{T}{S} { yylval.int_val = CMD_TS; return(CMD); }
{T}{T} { yylval.int_val = CMD_TT; return(CMD); }
{T}{V} { yylval.int_val = CMD_TV; return(CMD); }
{T}{Y} { yylval.int_val = CMD_TY; return(CMD); }
{T}{Z} { yylval.int_val = CMD_TZ; return(CMD); }
{V}{E} { yylval.int_val = CMD_VE; return(CMD); }

{E}{M} { yylval.int_val = CMD_EM; return(CMD); }
{M}{C} { yylval.int_val = CMD_MC; return(CMD); }
{M}{D} { yylval.int_val = CMD_MD; return(CMD); }
{R}{M} { yylval.int_val = CMD_RM; return(CMD); }
{R}{P} { yylval.int_val = CMD_RP; return(CMD); }
{T}{M} { yylval.int_val = CMD_TM; return(CMD); }
{W}{A} { yylval.int_val = CMD_WA; return(CMD); }
{W}{C} { yylval.int_val = CMD_WC; return(CMD); }
{W}{E} { yylval.int_val = CMD_WE; return(CMD); }
{W}{F} { yylval.int_val = CMD_WF; return(CMD); }
{W}{N} { yylval.int_val = CMD_WN; return(CMD); }
{W}{P} { yylval.int_val = CMD_WP; return(CMD); }
{W}{S} { yylval.int_val = CMD_WS; return(CMD); }
{X}{F} { yylval.int_val = CMD_XF; return(CMD); }
{X}{N} { yylval.int_val = CMD_XN; return(CMD); }

\$     { yylval.int_val = CMD_TA; return(CMD); }
\%     { yylval.int_val = CMD_TS; return(CMD); }
\&     { yylval.int_val = CMD_TV; return(CMD); }
\#     { yylval.int_val = CMD_TC; return(CMD); }
\@     { yylval.int_val = CMD_TP; return(CMD); }
\+     { yylval.int_val = CMD_TE; return(CMD); }


{H}{M} { BEGIN HEXMODE;
		 yylval.int_val = CMD_HM; return(CMD); }
{D}{M} { BEGIN DECMODE;
		 yylval.int_val = CMD_DM; return(CMD); }


<DECMODE,INITIAL>-?[[:digit:]]+/{ENDCHAR} {
	yylval.long_val = strtol(yytext, 0, 10);
	return(PARAM);
}

<HEXMODE>-?[[:xdigit:]]+/{ENDCHAR} {
	yylval.long_val = strtol(yytext, 0, 16);
	return(PARAM);
}


, {
	yylval.int_val = CMDEND;
	return(CMDEND);
}


\r {
	yylval.int_val = SEQEND;
	return(SEQEND);
}

[ \t] {
	/* ignore */;
}

. {
	return(PERROR);
}

%%

